@@ -1,7 +1,10 @@
+# Licensed under the Apache License: http://www.apache.org/licenses/LICENSE-2.0
+# For details: https://bitbucket.org/ned/coveragepy/src/default/NOTICE.txt
+
 copy("src", "out")
 run("""
-    coverage -e -x xxx
-    coverage -r
+    coverage run xxx
+    coverage report
     """, rundir="out", outfile="stdout.txt")
 contains("out/stdout.txt",
         "xxx: 3 4 0 7",
