@@ -14,7 +14,11 @@ import sys
 import coverage
 from coverage import env
 from coverage.backward import StringIO
-from coverage.misc import output_encoding
+from coverage.config import CoverageConfig
+from coverage.control import Coverage
+from coverage.data import CoverageData
+from coverage.misc import CoverageException, output_encoding
+from coverage.summary import SummaryReporter
 
 from tests.coveragetest import CoverageTest
 
@@ -416,6 +420,7 @@ class SummaryTest(CoverageTest):
             u"Couldn't parse 'accented\xe2.py' as Python source: 'error' at line 1"
         )
         if env.PY2:
+            # pylint: disable=redefined-variable-type
             expected = expected.encode(output_encoding())
         self.assertEqual(last, expected)
 
@@ -473,7 +478,7 @@ class SummaryTest(CoverageTest):
             """)
         cov = coverage.Coverage(branch=True, source=["."])
         cov.start()
-        import main     # pragma: nested # pylint: disable=import-error,unused-variable
+        import main     # pragma: nested # pylint: disable=import-error
         cov.stop()      # pragma: nested
         report = self.get_report(cov).splitlines()
         self.assertIn("mybranch.py 5 5 2 0 0%", report)
@@ -482,7 +487,7 @@ class SummaryTest(CoverageTest):
         """A helper for the next few tests."""
         cov = coverage.Coverage()
         cov.start()
-        import TheCode  # pragma: nested # pylint: disable=import-error,unused-variable
+        import TheCode  # pragma: nested # pylint: disable=import-error
         cov.stop()      # pragma: nested
         return self.get_report(cov)
 
@@ -505,7 +510,7 @@ class SummaryTest(CoverageTest):
 
     def test_pyw_files(self):
         if not env.WINDOWS:
-            self.skip(".pyw files are only on Windows.")
+            self.skipTest(".pyw files are only on Windows.")
 
         # https://bitbucket.org/ned/coveragepy/issue/261
         self.make_file("start.pyw", """\
@@ -517,7 +522,7 @@ class SummaryTest(CoverageTest):
             """)
         cov = coverage.Coverage()
         cov.start()
-        import start    # pragma: nested # pylint: disable=import-error,unused-variable
+        import start    # pragma: nested # pylint: disable=import-error
         cov.stop()      # pragma: nested
 
         report = self.get_report(cov)
@@ -537,7 +542,7 @@ class SummaryTest(CoverageTest):
         # Run the program.
         cov = coverage.Coverage()
         cov.start()
-        import main     # pragma: nested # pylint: disable=import-error,unused-variable
+        import main     # pragma: nested # pylint: disable=import-error
         cov.stop()      # pragma: nested
 
         report = self.get_report(cov).splitlines()
@@ -546,7 +551,7 @@ class SummaryTest(CoverageTest):
     def test_missing_py_file_during_run(self):
         # PyPy2 doesn't run bare .pyc files.
         if env.PYPY and env.PY2:
-            self.skip("PyPy2 doesn't run bare .pyc files")
+            self.skipTest("PyPy2 doesn't run bare .pyc files")
 
         # Create two Python files.
         self.make_file("mod.py", "a = 1\n")
@@ -567,7 +572,7 @@ class SummaryTest(CoverageTest):
         # Run the program.
         cov = coverage.Coverage()
         cov.start()
-        import main     # pragma: nested # pylint: disable=import-error,unused-variable
+        import main     # pragma: nested # pylint: disable=import-error
         cov.stop()      # pragma: nested
 
         # Put back the missing Python file.
@@ -595,7 +600,7 @@ class SummaryTest2(CoverageTest):
         # statements, not one statement.
         cov = coverage.Coverage(branch=True)
         cov.start()
-        import usepkgs  # pragma: nested # pylint: disable=import-error,unused-variable
+        import usepkgs  # pragma: nested # pylint: disable=import-error
         cov.stop()      # pragma: nested
 
         repout = StringIO()
@@ -640,3 +645,112 @@ class ReportingReturnValueTest(CoverageTest):
         cov = self.run_coverage()
         val = cov.xml_report(include="*/doit.py")
         self.assertAlmostEqual(val, 85.7, 1)
+
+
+class TestSummaryReporterConfiguration(CoverageTest):
+    """Tests of SummaryReporter."""
+
+    run_in_temp_dir = False
+
+    # We just need some readable files to work with. These will do.
+    HERE = os.path.dirname(__file__)
+
+    LINES_1 = {
+        os.path.join(HERE, "test_api.py"): dict.fromkeys(range(300)),
+        os.path.join(HERE, "test_backward.py"): dict.fromkeys(range(20)),
+        os.path.join(HERE, "test_coverage.py"): dict.fromkeys(range(15)),
+    }
+
+    def get_coverage_data(self, lines):
+        """Get a CoverageData object that includes the requested lines."""
+        data = CoverageData()
+        data.add_lines(lines)
+        return data
+
+    def get_summary_text(self, coverage_data, options):
+        """Get text output from the SummaryReporter."""
+        cov = Coverage()
+        cov.start()
+        cov.stop()              # pragma: nested
+        cov.data = coverage_data
+        printer = SummaryReporter(cov, options)
+        destination = StringIO()
+        printer.report([], destination)
+        return destination.getvalue()
+
+    def test_test_data(self):
+        # We use our own test files as test data. Check that our assumptions
+        # about them are still valid.  We want the three columns of numbers to
+        # sort in three different orders.
+        data = self.get_coverage_data(self.LINES_1)
+        report = self.get_summary_text(data, CoverageConfig())
+        print(report)
+        # Name                     Stmts   Miss  Cover
+        # --------------------------------------------
+        # tests/test_api.py          339    155    54%
+        # tests/test_backward.py      13      3    77%
+        # tests/test_coverage.py     234    228     3%
+        # --------------------------------------------
+        # TOTAL                      586    386    34%
+
+        lines = report.splitlines()[2:-2]
+        self.assertEqual(len(lines), 3)
+        nums = [list(map(int, l.replace('%', '').split()[1:])) for l in lines]
+        # [
+        #  [339, 155, 54],
+        #  [ 13,   3, 77],
+        #  [234, 228,  3]
+        # ]
+        self.assertTrue(nums[1][0] < nums[2][0] < nums[0][0])
+        self.assertTrue(nums[1][1] < nums[0][1] < nums[2][1])
+        self.assertTrue(nums[2][2] < nums[0][2] < nums[1][2])
+
+    def test_defaults(self):
+        """Run the report with no configuration options."""
+        data = self.get_coverage_data(self.LINES_1)
+        opts = CoverageConfig()
+        report = self.get_summary_text(data, opts)
+        self.assertNotIn('Missing', report)
+        self.assertNotIn('Branch', report)
+
+    def test_print_missing(self):
+        """Run the report printing the missing lines."""
+        data = self.get_coverage_data(self.LINES_1)
+        opts = CoverageConfig()
+        opts.from_args(show_missing=True)
+        report = self.get_summary_text(data, opts)
+        self.assertIn('Missing', report)
+        self.assertNotIn('Branch', report)
+
+    def assert_ordering(self, text, *words):
+        """Assert that the `words` appear in order in `text`."""
+        indexes = list(map(text.find, words))
+        self.assertEqual(
+            indexes, sorted(indexes),
+            "The words %r don't appear in order in %r" % (words, text)
+        )
+
+    def test_sort_report_by_stmts(self):
+        # Sort the text report by the Stmts column.
+        data = self.get_coverage_data(self.LINES_1)
+        opts = CoverageConfig()
+        opts.from_args(sort='Stmts')
+        report = self.get_summary_text(data, opts)
+        self.assert_ordering(report, "test_backward.py", "test_coverage.py", "test_api.py")
+
+    def test_sort_report_by_cover(self):
+        # Sort the text report by the Cover column.
+        data = self.get_coverage_data(self.LINES_1)
+        opts = CoverageConfig()
+        opts.from_args(sort='Cover')
+        report = self.get_summary_text(data, opts)
+        self.assert_ordering(report, "test_coverage.py", "test_api.py", "test_backward.py")
+
+    def test_sort_report_by_invalid_option(self):
+        # Sort the text report by a nonsense column.
+        data = self.get_coverage_data(self.LINES_1)
+        opts = CoverageConfig()
+        opts.from_args(sort='Xyzzy')
+        msg = "Invalid sorting option: 'Xyzzy'"
+        with self.assertRaisesRegex(CoverageException, msg):
+            self.get_summary_text(data, opts)
